@@ -18,9 +18,11 @@ from typing import Any, Mapping, Optional, Type, Union
 
 import pandas as pd
 import tensorflow as tf
+from log_with_context import Logger
 
 import scalarstop.pickle
 from scalarstop._filesystem import rmtree
+from scalarstop._logging import Timeblock
 from scalarstop._naming import temporary_filename
 from scalarstop._single_namespace import SingleNamespace
 from scalarstop.dataclasses import asdict
@@ -29,6 +31,7 @@ from scalarstop.exceptions import (
     ElementSpecNotFound,
     FileExists,
     FileExistsDuringDataBlobCreation,
+    InconsistentCachingParameters,
     IsNotImplemented,
     TensorFlowDatasetNotFound,
 )
@@ -39,6 +42,8 @@ from scalarstop.hyperparams import (
     enforce_dict,
 )
 
+_LOGGER = Logger(__name__)
+
 _METADATA_JSON_FILENAME = "metadata.json"
 _METADATA_PICKLE_FILENAME = "metadata.pickle"
 _ELEMENT_SPEC_FILENAME = "element_spec.pickle"
@@ -331,21 +336,121 @@ class DataBlob(SingleNamespace):
             self._test = self.set_test()
         return self._test
 
-    def batch(self, batch_size: int, *, with_tf_distribute: bool = False) -> "DataBlob":
-        """Batch this :py:class:`DataBlob`."""
+    def batch(
+        self,
+        batch_size: int,
+        *,
+        training: bool = True,
+        validation: bool = True,
+        test: bool = True,
+        with_tf_distribute: bool = False,
+    ) -> "DataBlob":
+        """
+        Batch this :py:class:`DataBlob`.
+
+        Args:
+            batch_size: The number of items to collect into a batch.
+
+            training: Whether to batch the training set.
+                Defaults to ``True``.
+
+            validation: Whether to batch the validation set.
+                Defaults to ``True``.
+
+            test: Whether to batch the test set. Defaults to ``True``.
+
+            with_tf_distribute: Whether to consider ``tf.distribute``
+                auto-data sharding when calculating the batch size.
+
+        """
         return _BatchDataBlob(
-            wraps=self, batch_size=batch_size, with_tf_distribute=with_tf_distribute
+            wraps=self,
+            batch_size=batch_size,
+            training=training,
+            validation=validation,
+            test=test,
+            with_tf_distribute=with_tf_distribute,
         )
 
-    def cache(self) -> "DataBlob":
-        """Cache this :py:class:`DataBlob` into memory before iterating over it."""
-        return _CacheDataBlob(wraps=self)
+    def cache(
+        self,
+        *,
+        training: bool = True,
+        validation: bool = True,
+        test: bool = True,
+        precache_training: bool = False,
+        precache_validation: bool = False,
+        precache_test: bool = False,
+    ) -> "DataBlob":
+        """
+        Cache this :py:class:`DataBlob` into memory before iterating over it.
 
-    def with_options(self, options: tf.data.Options) -> "DataBlob":
+        By default, this creates a :py:class:`DataBlob` containing a
+        TensorFlow ``CacheDataset`` for each of the training, validation and test
+        :py:class:`tf.data.Dataset` s.
+
+        But these datasets do not load into memory until the first time you
+        *completely* iterate over one--from start to end. If you want to
+        immediately load your training, validation, or test sets, you can
+        set ``precache_training``, ``precache_validation``, and/or
+        ``precache_test`` to ``True``.
+
+        Args:
+            training: Lazily cache the training set in CPU memory.
+                Defaults to ``True``.
+
+            validation: Lazily cache the validation set in CPU memory.
+                Defaults to ``True``.
+
+            test: Lazily cache the test set in CPU memory.
+                Defaults to ``True``.
+
+            precache_training: Eagerly cache the training set into memory.
+                Defaults to ``False``.
+
+            precache_validation: Eagerly cache the validation set into
+                memory. Defaults to ``False``.
+
+            precache_test: Eagerly cache the test set into memory.
+                Defaults to ``False``.
+        """
+        return _CacheDataBlob(
+            wraps=self,
+            training=training,
+            validation=validation,
+            test=test,
+            precache_training=precache_training,
+            precache_validation=precache_validation,
+            precache_test=precache_test,
+        )
+
+    def with_options(
+        self,
+        options: tf.data.Options,
+        *,
+        training: bool = True,
+        validation: bool = True,
+        test: bool = True,
+    ) -> "DataBlob":
         """
         Apply a :py:class:`tf.data.Options` object to this :py:class:`DataBlob`.
+
+        Args:
+            options: The :py:class:`tf.data.Options` object to apply.
+
+            training: Apply the options to the training set. Defaults to ``True``.
+
+            validation: Apply the options to the validation set. Defaults to ``True``.
+
+            test: Apply the options to the test set. Defaults to ``True``.
         """
-        return _WithOptionsDataBlob(wraps=self, options=options)
+        return _WithOptionsDataBlob(
+            wraps=self,
+            options=options,
+            training=training,
+            validation=validation,
+            test=test,
+        )
 
     def save_hook(  # pylint: disable=unused-argument
         self, *, subtype: str, path: str
@@ -681,12 +786,17 @@ class _WrapDataBlob(DataBlob):
     # We do not call super().__init__() because that call would be affected
     # by our custom __getattribute__(). Thankfully, our __getattribute__()
     # takes care of any setup that super().__init__() would handle.
-    def __init__(self, wraps: DataBlob):  # pylint: disable=super-init-not-called
+    def __init__(
+        self, wraps: DataBlob, *, training: bool, validation: bool, test: bool
+    ):  # pylint: disable=super-init-not-called
         self._wraps = wraps
         self.Hyperparams = self._wraps.Hyperparams
         self.hyperparams = self._wraps.hyperparams
         self._name = wraps.name
         self._group_name = wraps.group_name
+        self._enable_training = training
+        self._enable_validation = validation
+        self._enable_test = test
 
     def __getattribute__(self, key: str):
         """
@@ -705,33 +815,39 @@ class _WrapDataBlob(DataBlob):
         raise IsNotImplemented("_WrapDataBlob._wrap_tfdata()")
 
     def set_training(self) -> tf.data.Dataset:
-        return self._wrap_tfdata(self._wraps.set_training())
+        if self._enable_training:
+            return self._wrap_tfdata(self._wraps.set_training())
+        return self._wraps.set_training()
 
     @property
     def training(self) -> tf.data.Dataset:
         """An instance of the training set tf.data."""
         if self._training is None:
-            self._training = self._wrap_tfdata(self._wraps.training)
+            self._training = self.set_training()
         return self._training
 
     def set_validation(self) -> tf.data.Dataset:
-        return self._wrap_tfdata(self._wraps.set_validation())
+        if self._enable_validation:
+            return self._wrap_tfdata(self._wraps.set_validation())
+        return self._wraps.set_validation()
 
     @property
     def validation(self) -> tf.data.Dataset:
         """An instance of the validation set tf.data."""
         if self._validation is None:
-            self._validation = self._wrap_tfdata(self._wraps.validation)
+            self._validation = self.set_validation()
         return self._validation
 
     def set_test(self) -> tf.data.Dataset:
-        return self._wrap_tfdata(self._wraps.set_test())
+        if self._enable_test:
+            return self._wrap_tfdata(self._wraps.set_test())
+        return self._wraps.set_test()
 
     @property
     def test(self) -> tf.data.Dataset:
         """An instance of the test set tf.data."""
         if self._test is None:
-            self._test = self._wrap_tfdata(self._wraps.test)
+            self._test = self.set_test()
         return self._test
 
     def save_hook(self, *, subtype: str, path: str) -> None:
@@ -925,9 +1041,18 @@ class _BatchDataBlob(_WrapDataBlob):
     """
 
     def __init__(
-        self, *, wraps: Any, batch_size: int, with_tf_distribute: bool = False
+        self,
+        *,
+        wraps: Any,
+        batch_size: int,
+        training: bool,
+        validation: bool,
+        test: bool,
+        with_tf_distribute: bool = False,
     ):
-        super().__init__(wraps=wraps)
+        super().__init__(
+            wraps=wraps, training=training, validation=validation, test=test
+        )
         self._input_batch_size = batch_size
         self._with_tf_distribute = with_tf_distribute
         if self._with_tf_distribute:
@@ -947,11 +1072,77 @@ class _BatchDataBlob(_WrapDataBlob):
         return tfdata.batch(self._final_batch_size)
 
 
+def _precache_tfdata(tfdata: tf.data.Dataset) -> tf.data.Dataset:
+    """Tensorflow CacheDatasets will internally cache themselves if you iterate
+    over them."""
+    for _ in tfdata:
+        continue
+    return tfdata
+
+
 class _CacheDataBlob(_WrapDataBlob):
     """
     Cache this :py:class:`DataBlob` in memory.
     """
 
+    def __init__(
+        self,
+        *,
+        wraps: Any,
+        training: bool,
+        validation: bool,
+        test: bool,
+        precache_training: bool,
+        precache_validation: bool,
+        precache_test: bool,
+    ):
+        if training is False and precache_training is True:
+            raise InconsistentCachingParameters(
+                "You cannot pass `training=False` and `precache_training=True` "
+                "to `DataBlob.cache()`. If you want `precache_training=True`, "
+                "then set `training=True`, which it is by default."
+            )
+        if validation is False and precache_validation is True:
+            raise InconsistentCachingParameters(
+                "You cannot pass `validation=False` and `precache_validation=True` "
+                "to `DataBlob.cache()`. If you want `precache_validation=True`, "
+                "then set `validation=True`, which it is by default."
+            )
+        if test is False and precache_test is True:
+            raise InconsistentCachingParameters(
+                "You cannot pass `test=False` and `precache_test=True` "
+                "to `DataBlob.cache()`. If you want `precache_test=True`, "
+                "then set `test=True`, which it is by default."
+            )
+        super().__init__(
+            wraps=wraps, training=training, validation=validation, test=test
+        )
+        self._precache_training = precache_training
+        self._precache_validation = precache_validation
+        self._precache_test = precache_test
+
+        if self._precache_training:
+            with Timeblock(
+                name=f"(pre)caching {self.name}/training", print_function=_LOGGER.info
+            ):
+                self._training = _precache_tfdata(
+                    self._wrap_tfdata(self._wraps.training)
+                )
+
+        if self._precache_validation:
+            with Timeblock(
+                name=f"(pre)caching {self.name}/validation", print_function=_LOGGER.info
+            ):
+                self._validation = _precache_tfdata(
+                    self._wrap_tfdata(self._wraps.validation)
+                )
+
+        if self._precache_test:
+            with Timeblock(
+                name=f"(pre)caching {self.name}/validation", print_function=_LOGGER.info
+            ):
+                self._test = _precache_tfdata(self._wrap_tfdata(self._wraps.test))
+
     def _wrap_tfdata(self, tfdata: tf.data.Dataset) -> tf.data.Dataset:
         return tfdata.cache()
 
@@ -966,8 +1157,13 @@ class _WithOptionsDataBlob(_WrapDataBlob):
         *,
         wraps: Any,
         options: tf.data.Options,
+        training: bool,
+        validation: bool,
+        test: bool,
     ):
-        super().__init__(wraps=wraps)
+        super().__init__(
+            wraps=wraps, training=training, validation=validation, test=test
+        )
         self._tfdata_options = options
 
     def _wrap_tfdata(self, tfdata: tf.data.Dataset) -> tf.data.Dataset:
